-- Class for cast bars

bCastBar = {}
bCastBar.__index = bCastBar

function bCastBar.New(unit)
	local cBar = {}
	setmetatable(cBar, bCastBar)
	
	cBar.unit = unit
	cBar.locked = true
	cBar.fadeOutTimer = 0
	cBar.timeRemaining = 0
	
	cBar.config = bCast_profiles
	cBar.curProf = bCast_currentProfile
	
	cBar.context = UI.CreateContext(cBar.unit .. " context")
	
	-- Base invisible anchor frame
	cBar.bFrame = UI.CreateFrame("Frame", "bCast.bFrame", cBar.context)
	
	-- Bar texture frame
	cBar.bFrame.BarTex = UI.CreateFrame("Texture", "cBar.BarTex", cBar.bFrame)
	cBar.bFrame.BarTex:SetTexture("bCast", "media\\" .. cBar.config[cBar.curProf][cBar.unit].texture .. ".tga")
	
	-- Background frame
	cBar.bFrame.Background = UI.CreateFrame("Texture", "cBar.Background", cBar.bFrame)
	cBar.bFrame.Background:SetBackgroundColor(cBar.config[cBar.curProf][cBar.unit].colors.bar_r, cBar.config[cBar.curProf][cBar.unit].colors.bar_g, cBar.config[cBar.curProf][cBar.unit].colors.bar_b)
	cBar.bFrame.Background:SetAlpha(0.3)
	
	-- Bar color frame
	cBar.bFrame.Bar = UI.CreateFrame("Frame", "cBar.Bar", cBar.bFrame)
	cBar.bFrame.Bar:SetBackgroundColor(cBar.config[cBar.curProf][cBar.unit].colors.bar_r, cBar.config[cBar.curProf][cBar.unit].colors.bar_g, cBar.config[cBar.curProf][cBar.unit].colors.bar_b)
	cBar.bFrame.Bar:SetAlpha(0.6)
	
	-- Cast name & shadow
	cBar.bFrame.SpellText = UI.CreateFrame("Text", "cBar.SpellText", cBar.bFrame)
	cBar.bFrame.SpellText:SetFontColor(cBar.config[cBar.curProf][cBar.unit].colors.text_r, cBar.config[cBar.curProf][cBar.unit].colors.text_g, cBar.config[cBar.curProf][cBar.unit].colors.text_b)
	cBar.bFrame.SpellTextShadow = UI.CreateFrame("Text", "cBar.SpellTextShadow", cBar.bFrame)
	cBar.bFrame.SpellTextShadow:SetFontColor(0,0,0)
	cBar.bFrame.SpellText:SetText("???")
	cBar.bFrame.SpellTextShadow:SetText("???")
	
	-- Cast timer
	cBar.bFrame.TimeText = UI.CreateFrame("Text", "cBar.TimeText", cBar.bFrame)
	cBar.bFrame.TimeText:SetFontColor(cBar.config[cBar.curProf][cBar.unit].colors.text_r, cBar.config[cBar.curProf][cBar.unit].colors.text_g, cBar.config[cBar.curProf][cBar.unit].colors.text_b)
	cBar.bFrame.TimeTextShadow = UI.CreateFrame("Text", "cBar.TimeTextShadow", cBar.bFrame)
	cBar.bFrame.TimeTextShadow:SetFontColor(0,0,0)
	cBar.bFrame.TimeText:SetText("10.00/10.00")
	cBar.bFrame.TimeTextShadow:SetText("10.00/10.00")
	
	-- Set all the layers
	cBar.bFrame.TimeText:SetLayer(4)			-- Cast time
	cBar.bFrame.TimeTextShadow:SetLayer(3)		-- Cast time shadow
	cBar.bFrame.SpellText:SetLayer(4)			-- Spell name
	cBar.bFrame.SpellTextShadow:SetLayer(3)		-- Spell name shadow
	cBar.bFrame.Bar:SetLayer(2)					-- Cast bar color (solid)
	cBar.bFrame.BarTex:SetLayer(1) 				-- Cast bar texture
	cBar.bFrame:SetLayer(0)						-- Invisible anchor frame
	cBar.bFrame.Background:SetLayer(-1)			-- Background solid
	
	-- Cast icon
	cBar.Icon = UI.CreateFrame("Texture", "cBar.Icon", cBar.bFrame)
	cBar.Icon:SetWidth(cBar.config[cBar.curProf][cBar.unit].height)
	cBar.Icon:SetHeight(cBar.config[cBar.curProf][cBar.unit].height)
	cBar.Icon:SetLayer(5)
	
	cBar.bFrame:SetVisible(false)
	
	local mDown = false
	local mouse = nil
	local mPosX = 0
	local mPosY = 0
	
	function cBar.bFrame.Event.LeftDown()
		if not cBar.locked then
			mDown = true
			mouse = Inspect.Mouse()
			mPosX = mouse.x - cBar.bFrame:GetLeft()
			mPosY = mouse.y - cBar.bFrame:GetTop()
		end
	end
	
	function cBar.bFrame.Event.MouseMove()
		if not cBar.locked then
			if mDown then
				local newX = 0
				local newY = 0
				mouse = Inspect.Mouse()
				newX = mouse.x - mPosX
				newY = mouse.y - mPosY
				cBar.bFrame:SetPoint("TOPLEFT", UIParent, "TOPLEFT", newX, newY)
			end
		end
	end
	
	function cBar.bFrame.Event.LeftUp()
		if not cBar.locked then
			if mDown then
				mDown = false
			end
			cBar.config[cBar.curProf][cBar.unit].xPos = cBar.bFrame:GetLeft()
			cBar.config[cBar.curProf][cBar.unit].yPos = cBar.bFrame:GetTop()
		end
	end
	
	return cBar
end

-- toggles bar locking for moving
function bCastBar:Lock(lock)
	self.locked = lock
	self:SetVisible(not self.locked)
end

-- Set the colors for interrupt fadeout
function bCastBar:FadeOutColor()
	-- To fade the bar on interrupt to actual bar color, instead of default
	self.interruptRedDiff = (1-bCast_profiles[bCast_currentProfile][self.unit].colors.bar_r)/14
	self.interruptGreenDiff = (bCast_profiles[bCast_currentProfile][self.unit].colors.bar_g)/14
	self.interruptBlueDiff = (bCast_profiles[bCast_currentProfile][self.unit].colors.bar_b)/14
	
	self.currentRed = 1
	self.currentGreen = 0
	self.currentBlue = 0
end


-- Resizes, repositions etc all the cast bar frames
function bCastBar:UpdateFrames()

	self.config = bCast_profiles
	self.curProf = bCast_currentProfile

	-- all the different bars
	self.bFrame:SetPoint("TOPLEFT", UIParent, "TOPLEFT", self.config[self.curProf][self.unit].xPos, self.config[self.curProf][self.unit].yPos)
	self.bFrame:SetWidth(self.config[self.curProf][self.unit].width)
	self.bFrame:SetHeight(self.config[self.curProf][self.unit].height)
	self.bFrame.BarTex:SetPoint("TOPLEFT", self.bFrame, "TOPLEFT")
	self.bFrame.BarTex:SetWidth(self.config[self.curProf][self.unit].width)
	self.bFrame.BarTex:SetHeight(self.config[self.curProf][self.unit].height)
	self.bFrame.BarTex:SetTexture("bCast", "media\\" .. self.config[self.curProf][self.unit].texture .. ".tga")
	self.bFrame.Background:SetPoint("TOPLEFT", self.bFrame, "TOPLEFT")
	self.bFrame.Background:SetWidth(self.config[self.curProf][self.unit].width)
	self.bFrame.Background:SetHeight(self.config[self.curProf][self.unit].height)
	self.bFrame.Bar:SetPoint("TOPLEFT", self.bFrame, "TOPLEFT")
	self.bFrame.Bar:SetWidth(self.config[self.curProf][self.unit].width)
	self.bFrame.Bar:SetHeight(self.config[self.curProf][self.unit].height)
	
	-- spell name texts
	self.bFrame.SpellText:SetFont("bCast", "media\\" .. self.config[self.curProf][self.unit].font ..".ttf")
	self.bFrame.SpellTextShadow:SetFont("bCast", "media\\" .. self.config[self.curProf][self.unit].font ..".ttf")
	self.bFrame.SpellText:SetFontSize(self.config[self.curProf][self.unit].fontSize)
	self.bFrame.SpellTextShadow:SetFontSize(self.config[self.curProf][self.unit].fontSize)
	
	self.bFrame.SpellText:SetWidth(self.config[self.curProf][self.unit].width)
	self.bFrame.SpellTextShadow:SetWidth(self.config[self.curProf][self.unit].width)	
	
	self.bFrame.SpellText:SetPoint("CENTERLEFT", self.bFrame, "CENTERLEFT")
	self.bFrame.SpellTextShadow:SetPoint("CENTERLEFT", self.bFrame, "CENTERLEFT", 1, 2)
	
	self.bFrame.SpellText:SetFontColor(self.config[self.curProf][self.unit].colors.text_r, self.config[self.curProf][self.unit].colors.text_g, self.config[self.curProf][self.unit].colors.text_b)
	
	-- cast time texts
	self.bFrame.TimeText:SetFont("bCast", "media\\" .. self.config[self.curProf][self.unit].font ..".ttf")
	self.bFrame.TimeTextShadow:SetFont("bCast", "media\\" .. self.config[self.curProf][self.unit].font ..".ttf")
	self.bFrame.TimeText:SetFontSize(self.config[self.curProf][self.unit].fontSize)
	self.bFrame.TimeTextShadow:SetFontSize(self.config[self.curProf][self.unit].fontSize)
		
	self.bFrame.TimeText:SetPoint("CENTERRIGHT", self.bFrame, "CENTERRIGHT")
	self.bFrame.TimeTextShadow:SetPoint("CENTERRIGHT", self.bFrame, "CENTERRIGHT", 1, 2)
	
	self.bFrame.TimeText:SetFontColor(self.config[self.curProf][self.unit].colors.text_r, self.config[self.curProf][self.unit].colors.text_g, self.config[self.curProf][self.unit].colors.text_b)
	
	-- ability icon
	self.Icon:ClearAll()
	self.Icon:SetWidth(self.config[self.curProf][self.unit].height)
	self.Icon:SetHeight(self.config[self.curProf][self.unit].height)
	if self.config[self.curProf][self.unit].icon == "left" then
		self.Icon:SetPoint("CENTERRIGHT", self.bFrame, "CENTERLEFT", -2, 0)
	else -- if self.config[self.curProf][self.unit].icon == "right" then
		self.Icon:SetPoint("CENTERLEFT", self.bFrame, "CENTERRIGHT", 2, 0)
	end
	
	if self.config[self.curProf][self.unit].icon == "none" then
		self.Icon:SetVisible(false)
	end
	
end

-- Sets the displayed ability name to both actual text and shadow, also truncates it if needed
function bCastBar:SetAbilityName(name)
	if name then
		if string.len(name) > (self.config[self.curProf][self.unit].maxLength + 3) then -- +3 so last three letters of ability name don't get replaced, when they would fit in the same space as "..."
			-- Truncate ability name if too long
			name = string.sub(name, 1, self.config[self.curProf][self.unit].maxLength)
			name = name .. "..."
		end
	else
		-- if castBar.abilityName returns nil for some odd reason
		name = "???"
	end
	self.bFrame.SpellText:SetText(name)
	self.bFrame.SpellTextShadow:SetText(name)
end

-- Called on cast start for this unit, resets the bar values for a new cast
-- Also sets the bar color, cast name and draws the icon if needed
function bCastBar:ResetValues()
	self.config = bCast_profiles
	self.curProf = bCast_currentProfile	
	self.fadeOutTimer = 0 
	if self.unit == "player" and self.config[self.curProf].player.pushBack then
		-- should fix case where pushback from previous cast carried over to next
		self.originalDuration = 0
		self.pushBackDiff = 0
	end
	
	if self.config[self.curProf][self.unit].icon == "none" then
		self.Icon:SetVisible(false)
	else
		self.Icon:SetVisible(true)
	end
	local castBar = Inspect.Unit.Castbar(self.unit)
	if castBar then
		if self.unit == "player" or not castBar.uninterruptible then
			-- Set color of a player cast bar or interruptible cast bar
			self.bFrame.Bar:SetBackgroundColor(self.config[self.curProf][self.unit].colors.bar_r, self.config[self.curProf][self.unit].colors.bar_g, self.config[self.curProf][self.unit].colors.bar_b) 
			self.bFrame.Background:SetBackgroundColor(self.config[self.curProf][self.unit].colors.bar_r, self.config[self.curProf][self.unit].colors.bar_g, self.config[self.curProf][self.unit].colors.bar_b) 
		else
			-- Set color of an uninterruptible cast bar
			self.bFrame.Bar:SetBackgroundColor(self.config[self.curProf][self.unit].colors.bar_ir, self.config[self.curProf][self.unit].colors.bar_ig, self.config[self.curProf][self.unit].colors.bar_ib) 
			self.bFrame.Background:SetBackgroundColor(self.config[self.curProf][self.unit].colors.bar_ir, self.config[self.curProf][self.unit].colors.bar_ig, self.config[self.curProf][self.unit].colors.bar_ib) 
		end
		self:SetAbilityName(castBar.abilityName)
		
		-- Draw the icon if needed
		if self.config[self.curProf][self.unit].icon ~= "none" then
			-- icon is enabled
			if castBar.ability then
				local ability = Inspect.Ability.Detail(castBar.ability)
				if ability then
					if ability.icon then
						-- Proper icon found, use it
						self.Icon:SetTexture("Rift", ability.icon)
					else
						-- in other cases, use the placeholder apple
						self.Icon:SetTexture("Rift", "apple_a.dds")
					end
				else
					self.Icon:SetTexture("Rift", "apple_a.dds")
				end
			else
				self.Icon:SetTexture("Rift", "apple_a.dds")
			end
		end
	end
	self.bFrame:SetVisible(true)
end

-- Animates the bar for casts
function bCastBar:Update()
	local castBar
	if not self.locked then -- Bar is unlocked for dragging, show a dummy bar
		castBar = {
			abilityName = self.unit .. " - Move me!",
			duration = 10,
			remaining = 0,
			channeled = false,
		}
		self:SetAbilityName(castBar.abilityName)
	else
		-- Look for normal cast bar
		castBar = Inspect.Unit.Castbar(self.unit)
	end
	
	if castBar then
		if not self.bFrame:GetVisible() then
			-- If this unit gets targeted mid cast, need to reset everything and set the bar visible
			self:ResetValues()
		end
		self.timeRemaining = castBar.remaining
		local currentProgress = self.timeRemaining/castBar.duration
		local timeText = ""
		
		local duration = castBar.duration
			
		if self.unit == "player" and self.config[self.curProf][self.unit].pushBack then
			-- For detailed push back duration view
			if (self.originalDuration == 0) then -- set original cast duration if no previous
				self.originalDuration = duration
			end
			
			if not (self.originalDuration == castBar.duration) then
				-- pushback detected
				self.pushBackDiff = castBar.duration - self.originalDuration
				if self.pushBackDiff > 0 then
					duration = string.format("%.2f", self.originalDuration) .. "+" .. string.format("%.2f", self.pushBackDiff)
				else
					duration = string.format("%.2f", self.originalDuration) .. string.format("%.2f", self.pushBackDiff)
				end
			else
				duration = string.format("%.2f", duration)
			end
		else
			duration = string.format("%.2f", duration)
		end
		
		-- set the castbar width during cast
		if not castBar.channeled then
			-- normal cast
			local nWidth = (1-currentProgress)*self.config[self.curProf][self.unit].width
			self.bFrame.Bar:SetWidth(nWidth)
			self.bFrame.BarTex:SetWidth(nWidth)
			if self.config[self.curProf][self.unit].timeStyle == "done" then
				timeText = string.format("%.2f/", castBar.duration-self.timeRemaining) .. duration
			else
				timeText = string.format("%.2f/", self.timeRemaining) .. duration
			end
		else
			-- reverse if channeled
			local nWidth = currentProgress*self.config[self.curProf][self.unit].width
			self.bFrame.Bar:SetWidth(nWidth)
			self.bFrame.BarTex:SetWidth(nWidth)
			timeText = string.format("%.2f/", self.timeRemaining) .. duration
		end
		-- update the cast time display
		self.bFrame.TimeText:SetText(timeText)
		self.bFrame.TimeTextShadow:SetText(timeText)
		
	else
		-- Cast has finished, inspecting the castbar returns nil
		if (self.timeRemaining < 0.05 and self.fadeOutTimer == 0) or not Inspect.Unit.Detail(self.unit) then -- cast finished normally, or this unit is no longer targeted
			self.bFrame:SetVisible(false)
		else -- Cast was interrupted
			if self.fadeOutTimer == 0 then -- start the fadeout animation
				self.fadeOutTimer = 14
				self.bFrame.Bar:SetBackgroundColor(1, 0, 0) -- set cast bar to red
				self.bFrame.Bar:SetWidth(self.config[self.curProf][self.unit].width)	-- set cast bar to full length
				self.bFrame.BarTex:SetWidth(self.config[self.curProf][self.unit].width)
				
				self.currentRed = 1
				self.currentGreen = 0
				self.currentBlue = 0
			else	-- loop this for the fadeout animation, until fadeOutTimer == 0
				self.fadeOutTimer = self.fadeOutTimer - 1
				self.currentRed = self.currentRed - self.interruptRedDiff -- substract, because color starts from pure red
				self.currentGreen = self.currentGreen + self.interruptGreenDiff
				self.currentBlue = self.currentBlue + self.interruptBlueDiff
				self.bFrame.Bar:SetBackgroundColor(self.currentRed, self.currentGreen, self.currentBlue)
			end
			self.timeRemaining = 0
		end
	end
end

function bCastBar:SetVisible(visible)
	self.bFrame:SetVisible(visible)
end