bCastOptions = {}
local opt = bCastOptions
local config = bCast_profiles
local curProf = bCast_currentProfile

local context = UI.CreateContext("Options Context")
local win = UI.CreateFrame("SimpleWindow", "bCastOptions", context)
local showOptions = false
local selected = "Player"

win:SetPoint("TOPLEFT", context, "TOPLEFT", 200, 200)
win:SetWidth(550)
win:SetHeight(650)
win.modeSelectList = UI.CreateFrame("SimpleList", "bCastOptionsModeList", win)
win.modeSelectList:SetPoint("TOPLEFT", win:GetContent(), "TOPLEFT", 5, 5)
win.modeSelectList:SetBorder(1, 1, 1, 1, 1)
win.modeSelectList:SetWidth(135)
win.modeSelectList:SetItems( {"Player", "Target", "Target of target", "Focus", "Misc" } )

win.lockButton = UI.CreateFrame("RiftButton", "bCastLockButton", win)
win.lockButton:SetText("Toggle lock")
win.lockButton:SetPoint("TOPLEFT", win.modeSelectList, "BOTTOMLEFT", 0, 0)

win.cancelButton = UI.CreateFrame("RiftButton", "bCastCancelButton", win)
win.cancelButton:SetText("Cancel")
win.cancelButton:SetPoint("BOTTOMLEFT", win:GetContent(), "BOTTOMLEFT", 5, -5)

win.okButton = UI.CreateFrame("RiftButton", "bCastOKButton", win)
win.okButton:SetText("OK")
win.okButton:SetPoint("BOTTOMLEFT", win.cancelButton, "TOPLEFT", 0, 0)

win.applyButton = UI.CreateFrame("RiftButton", "bCastApplyButton", win)
win.applyButton:SetText("Apply")
win.applyButton:SetPoint("BOTTOMLEFT", win.okButton, "TOPLEFT", 0, 0)

win.contentFrame = UI.CreateFrame("Frame", "bCastOptionsContent", win)
win.contentFrame:SetPoint("TOPLEFT", win.modeSelectList, "TOPRIGHT", 5, 0)
win.contentFrame:SetWidth(win:GetContent():GetWidth() - 15 - win.modeSelectList:GetWidth())
win.contentFrame:SetHeight(win:GetContent():GetHeight() - 10)
win.contentFrame:SetBackgroundColor(0, 0, 0, 0.5)

win.contentFrame.player = bCastOptionsPane.New(win.contentFrame, "player")
win.contentFrame.target = bCastOptionsPane.New(win.contentFrame, "player.target")
win.contentFrame.targettarget = bCastOptionsPane.New(win.contentFrame, "player.target.target")
win.contentFrame.focus = bCastOptionsPane.New(win.contentFrame, "focus")
win.contentFrame.misc = bCastMiscPane.New(win.contentFrame, opt)

-- chooses which options to show
function opt.ShowUnit(unit)
	win.contentFrame.player:SetVisible(false)
	win.contentFrame.target:SetVisible(false)
	win.contentFrame.targettarget:SetVisible(false)
	win.contentFrame.focus:SetVisible(false)
	win.contentFrame.misc:SetVisible(false)
	selected = unit
	if unit == "Player" then
		win.contentFrame.player:SetVisible(true)
	elseif unit == "Target" then
		win.contentFrame.target:SetVisible(true)
	elseif unit == "Target of target" then
		win.contentFrame.targettarget:SetVisible(true)
	elseif unit == "Focus" then
		win.contentFrame.focus:SetVisible(true)
	elseif unit == "Misc" then
		win.contentFrame.misc:SetVisible(true)
	else
		selected = "player"
		print("You should not see this! If you do, please report it to the author.")
		print("Unit called: " ..  tostring(unit))
	end
end

function opt.LoadSettings()
	win.contentFrame.player:LoadOptions()
	win.contentFrame.target:LoadOptions()
	win.contentFrame.targettarget:LoadOptions()
	win.contentFrame.focus:LoadOptions()
end

function opt.SaveSettings()
	win.contentFrame.player:SaveOptions()
	win.contentFrame.target:SaveOptions()
	win.contentFrame.targettarget:SaveOptions()
	win.contentFrame.focus:SaveOptions()
	bCastSettingsChange()
end


-- called to show or hide the window
function opt.Show()
	opt.ShowUnit(selected)
	showOptions = not showOptions
	opt.LoadSettings()
	win.contentFrame.misc:SetProfile()
	win:SetVisible(showOptions)
end

function opt.ChangeProfile(newProfile)
	bCast_currentProfile = newProfile
	curProf = newProfile
	opt.LoadSettings()
end

function win.Event.Close()
	opt.Show()
end

function win.cancelButton.Event:LeftPress()
	opt.Show()
end

function win.okButton.Event:LeftPress()
	opt.SaveSettings()
	opt.Show()
end

function win.applyButton.Event:LeftPress()
	opt.SaveSettings()
end

-- Toggle the bar lock and update the position sliders
function win.lockButton.Event:LeftPress()
	win.contentFrame.player:LoadPosition()
	win.contentFrame.target:LoadPosition()
	win.contentFrame.targettarget:LoadPosition()
	win.contentFrame.focus:LoadPosition()
	bCastLockToggle(not bCastIsLocked())
end

-- called when a mode is selected
function win.modeSelectList.Event.ItemSelect(item, value)
	opt.ShowUnit(value)
end

function opt.AddonLoad()
	win:SetVisible(false)
	win.modeSelectList:SetSelectedIndex(1)
	win:SetCloseButtonVisible(true)
	win.contentFrame.misc:PopulateProfiles()
	win:SetTitle("bCast " .. bCastVersion .. " Options")
end