-- Class for misc options

bCastMiscPane = {}
bCastMiscPane.__index = bCastMiscPane

function bCastMiscPane.New(parent, options)
	local pane = {}
	setmetatable(pane, bCastMiscPane)
	pane.parent = parent
	pane.opt = options
	
	pane.base = UI.CreateFrame("Frame", "MiscOptions", parent)
	pane.base:SetPoint("TOPLEFT", parent, "TOPLEFT")
	pane.base:SetWidth(parent:GetWidth())
	pane.base:SetHeight(parent:GetHeight())
	
	-- Reset current profile
	pane.resetText = UI.CreateFrame("Text", "ResetText", pane.base)
	pane.resetText:SetText("Reset current profile\nto default settings:")
	pane.resetText:SetPoint("TOPLEFT", pane.base, "TOPLEFT", 5, 5)	
	pane.resetButton = UI.CreateFrame("RiftButton", "ResetButton", pane.base)
	pane.resetButton:SetText("Reset")
	pane.resetButton:SetPoint("TOPRIGHT", pane.base, "TOPRIGHT", -115, 5)
	pane.resetButton:SetHeight(pane.resetText:GetHeight())
	
	-- Create new profile
	pane.newText = UI.CreateFrame("Text", "NewProfileText", pane.base)
	pane.newText:SetText("Create new profile:")
	pane.newText:SetPoint("TOPLEFT", pane.resetText, "BOTTOMLEFT", 0, 5)
	pane.newTField = UI.CreateFrame("RiftTextfield", "NewProfileTField", pane.base)
	pane.newTField:SetWidth(245) -- should be the same the sliders on the other panes default to
	pane.newTField:SetPoint("TOPRIGHT", pane.resetButton, "BOTTOMRIGHT", 110, 5)
	pane.newTField:SetBackgroundColor(0,0,0)
	pane.newButton = UI.CreateFrame("RiftButton", "NewProfileButton", pane.base)
	pane.newButton:SetText("Create")
	pane.newButton:SetPoint("TOPRIGHT", pane.newTField, "BOTTOMRIGHT", 5, 0)
	
	-- Select profile
	pane.profText = UI.CreateFrame("Text", "ChangeProfileText", pane.base)
	pane.profText:SetText("Current profile:")
	pane.profText:SetPoint("TOPLEFT", pane.newText, "BOTTOMLEFT", 0, pane.newButton:GetHeight())
	pane.profBox = UI.CreateFrame("SimpleSelect", "ProfileChangeBox", pane.base)
	pane.profBox:SetBorder(1,1,1,1,1)
	pane.profBox:ResizeToDefault()
	pane.profBox:SetWidth(245)
	pane.profBox:SetPoint("TOPRIGHT", pane.newButton, "BOTTOMRIGHT", -5, 0)
	pane.profBox:SetLayer(2)
	
	-- Delete profile button
	pane.deleteButton = UI.CreateFrame("RiftButton", "DeleteProfileButton", pane.base)
	pane.deleteButton:SetText("Delete")
	pane.deleteButton:SetPoint("TOPRIGHT", pane.profBox, "BOTTOMRIGHT", 5, 0)
	pane.deleteButton:SetLayer(1)
	
	function pane.resetButton.Event:LeftPress()
		bCast_profiles[bCast_currentProfile] = bCastDefaults()
		options.LoadSettings()
	end
	
	function pane.newButton.Event:LeftPress()
		local profileName = pane.newTField:GetText()
		if profileName then
			if profileName ~= "" then
				bCast_profiles[profileName] = bCastDefaults()
				pane:PopulateProfiles()
				pane.newTField:SetText("")
				pane.newTField:SetKeyFocus(false)
				pane.profBox:SetSelectedItem(profileName) -- also changes the profile globally
			end
		end
	end
	
	function pane.deleteButton.Event:LeftPress()
		pane:DestroyWorld()
	end
	
	-- called when the selected item in the profile selection box changes
	function pane.profBox.Event.ItemSelect(item,value,index)
		if value then -- might be set to nil when removing values from the list box
			if not bCastIsLocked() then bCastLockToggle(true) end
			options.ChangeProfile(value) -- sets global current profile and loads new profile settings into options
		end
	end
	
	return pane
end

-- make sure the correct profile is displayed when its changed through slash commands
function bCastMiscPane:SetProfile()
	self.profBox:SetSelectedItem(bCast_currentProfile)
end

-- Deletes the current profile, repopulates the profile list and selects default as the current profile (hopefully)
function bCastMiscPane:DestroyWorld()
	local profileName = self.profBox:GetSelectedItem()
	if not bCastIsLocked() then bCastLockToggle(true) end
	if profileName ~= "default" then -- don't allow removing the default profile
		bCast_currentProfile = "default"
		bCast_profiles[profileName] = nil -- remove the selected profile
		self:PopulateProfiles()
		self.profBox:SetSelectedItem("default") -- repopulating profile list might change the current profile as a result of indices changing, need to force it back to default
		self.opt.ChangeProfile("default")
	end
end

-- Creates the list of items to show in the profile list, based on bCast_profiles
function bCastMiscPane:PopulateProfiles()
	local items = {}
	for k,v in pairs(bCast_profiles) do 
		if k then -- just in case
			table.insert(items, k)
		end
	end
	self.profBox:SetItems(items)
	self.profBox:SetSelectedItem(bCast_currentProfile)
end

function bCastMiscPane:SetVisible(value)
	self.newTField:SetKeyFocus(false)
	self.base:SetVisible(value)
end