-- Class for options
-- Each unit has their own instance

bCastOptionsPane = {}
bCastOptionsPane.__index = bCastOptionsPane

function bCastOptionsPane.New(parent, unitName)
	local pane = {}
	setmetatable(pane, bCastOptionsPane)
	pane.unit = unitName
	pane.parent = parent
	
	pane.base = UI.CreateFrame("Frame", unitName .. "Options", parent)
	pane.base:SetPoint("TOPLEFT", parent, "TOPLEFT")
	pane.base:SetWidth(parent:GetWidth())
	pane.base:SetHeight(parent:GetHeight())
	
	-- Bar width text & slider
	pane.widthText = UI.CreateFrame("Text", unitName .. "WidthText", pane.base)
	pane.widthText:SetText("Width:")
	pane.widthText:SetPoint("TOPLEFT", pane.base, "TOPLEFT", 5, 5)	
	pane.widthSlider = UI.CreateFrame("SimpleSlider", unitName .. "WidthSlider", pane.base)
	pane.widthSlider:SetPoint("TOPRIGHT", pane.base, "TOPRIGHT", -5, 5)
	pane.widthSlider:SetRange(10, 1000)
	pane.widthText:SetHeight(pane.widthSlider:GetHeight())
	
	-- Bar height text & slider
	pane.heightText = UI.CreateFrame("Text", unitName .. "HeightText", pane.base)
	pane.heightText:SetText("Height:")
	pane.heightText:SetPoint("TOPLEFT", pane.widthText, "BOTTOMLEFT", 0, 5)	
	pane.heightSlider = UI.CreateFrame("SimpleSlider", unitName .. "HeightSlider", pane.base)
	pane.heightSlider:SetPoint("TOPRIGHT", pane.widthSlider, "BOTTOMRIGHT", 0, 5)
	pane.heightSlider:SetRange(10, 1000)
	pane.heightText:SetHeight(pane.heightSlider:GetHeight())
	
	-- Bar x position text & slider
	pane.xPosText = UI.CreateFrame("Text", unitName .. "XPositionText", pane.base)
	pane.xPosText:SetText("x Position:")
	pane.xPosText:SetPoint("TOPLEFT", pane.heightText, "BOTTOMLEFT", 0, 5)	
	pane.xPosSlider = UI.CreateFrame("SimpleSlider", unitName .. "XPositionSlider", pane.base)
	pane.xPosSlider:SetPoint("TOPRIGHT", pane.heightSlider, "BOTTOMRIGHT", 0, 5)
	pane.xPosSlider:SetRange(0, math.floor(UIParent:GetWidth() + 0.5))
	pane.xPosText:SetHeight(pane.xPosSlider:GetHeight())
	
	-- Bar y position text & slider
	pane.yPosText = UI.CreateFrame("Text", unitName .. "YPositionText", pane.base)
	pane.yPosText:SetText("y Position:")
	pane.yPosText:SetPoint("TOPLEFT", pane.xPosText, "BOTTOMLEFT", 0, 5)	
	pane.yPosSlider = UI.CreateFrame("SimpleSlider", unitName .. "YPositionSlider", pane.base)
	pane.yPosSlider:SetPoint("TOPRIGHT", pane.xPosSlider, "BOTTOMRIGHT", 0, 5)
	pane.yPosSlider:SetRange(0, math.floor(UIParent:GetHeight() + 0.5))
	pane.yPosText:SetHeight(pane.yPosSlider:GetHeight())
	
	-- Bar color texts & sliders
	pane.colorText = UI.CreateFrame("Text", unitName .. "ColorLabel", pane.base)
	pane.colorText:SetText("Bar color:")
	pane.colorText:SetPoint("TOPLEFT", pane.yPosText, "BOTTOMLEFT", 0, 5)
	pane.colorTextRed = UI.CreateFrame("Text", unitName .. "ColorRedLabel", pane.base)
	pane.colorTextRed:SetText("Red")
	pane.colorTextRed:SetPoint("TOPLEFT", pane.colorText, "BOTTOMLEFT", 0, 5)
	pane.colorSliderRed = UI.CreateFrame("SimpleSlider", unitName .. "RedSlider", pane.base)
	pane.colorSliderRed:SetPoint("TOPRIGHT", pane.yPosSlider, "BOTTOMRIGHT", 0, pane.colorText:GetHeight() + 10) -- empty space right above this, need to anchor to previous slider above that
	pane.colorSliderRed:SetRange(0,255)
	pane.colorTextRed:SetHeight(pane.colorSliderRed:GetHeight())
	pane.colorTextGreen = UI.CreateFrame("Text", unitName .. "ColorGreenLabel", pane.base)
	pane.colorTextGreen:SetText("Green")
	pane.colorTextGreen:SetPoint("TOPLEFT", pane.colorTextRed, "BOTTOMLEFT", 0, 5)
	pane.colorSliderGreen = UI.CreateFrame("SimpleSlider", unitName .. "GreenSlider", pane.base)
	pane.colorSliderGreen:SetPoint("TOPRIGHT", pane.colorSliderRed, "BOTTOMRIGHT", 0, 5)
	pane.colorSliderGreen:SetRange(0,255)
	pane.colorTextGreen:SetHeight(pane.colorSliderGreen:GetHeight())
	pane.colorTextBlue = UI.CreateFrame("Text", unitName .. "ColorBlueLabel", pane.base)
	pane.colorTextBlue:SetText("Blue")
	pane.colorTextBlue:SetPoint("TOPLEFT", pane.colorTextGreen, "BOTTOMLEFT", 0, 5)
	pane.colorSliderBlue = UI.CreateFrame("SimpleSlider", unitName .. "BlueSlider", pane.base)
	pane.colorSliderBlue:SetPoint("TOPRIGHT", pane.colorSliderGreen, "BOTTOMRIGHT", 0, 5)
	pane.colorSliderBlue:SetRange(0,255)
	pane.colorTextBlue:SetHeight(pane.colorSliderBlue:GetHeight())

	-- Bar texture selector
	pane.texText = UI.CreateFrame("Text", unitName .. "TextureText", pane.base)
	pane.texText:SetText("Texture:")
	pane.texText:SetPoint("TOPLEFT", pane.colorTextBlue, "BOTTOMLEFT", 0, 5)
	pane.texBox = UI.CreateFrame("SimpleSelect", unitName .. "TextureSelector", pane.base)
	pane.texBox:SetBorder(1,1,1,1,1)
	pane.texBox:ResizeToDefault()
	pane.texBox:SetWidth(pane.colorSliderBlue:GetWidth())
	pane.texBox:SetPoint("TOPRIGHT", pane.colorSliderBlue, "BOTTOMRIGHT", 0, 5)
	pane.texBox:SetItems(bCastTextures)
	pane.texText:SetHeight(pane.texBox:GetHeight())
	pane.texBox:SetLayer(4)
	
	-- Icon state
	pane.iconText = UI.CreateFrame("Text", unitName .. "IconText", pane.base)
	pane.iconText:SetText("Icon:")
	pane.iconText:SetPoint("TOPLEFT", pane.texText, "BOTTOMLEFT", 0, 5)
	pane.iconBox = UI.CreateFrame("SimpleSelect", unitName .. "IconBox", pane.base)
	pane.iconBox:SetBorder(1,1,1,1,1)
	pane.iconBox:ResizeToDefault()
	pane.iconBox:SetWidth(pane.colorSliderBlue:GetWidth())
	pane.iconBox:SetPoint("TOPRIGHT", pane.texBox, "BOTTOMRIGHT", 0, 5)
	pane.iconBox:SetItems({"left","right","none"})
	pane.iconText:SetHeight(pane.iconBox:GetHeight())
	pane.iconBox:SetLayer(3)
	
	-- Font selector
	pane.fontText = UI.CreateFrame("Text", unitName .. "FontText", pane.base)
	pane.fontText:SetText("Font:")
	pane.fontText:SetPoint("TOPLEFT", pane.iconText, "BOTTOMLEFT", 0, 5)
	pane.fontBox = UI.CreateFrame("SimpleSelect", unitName .. "FontSelector", pane.base)
	pane.fontBox:SetBorder(1,1,1,1,1)
	pane.fontBox:ResizeToDefault()
	pane.fontBox:SetWidth(pane.colorSliderBlue:GetWidth())
	pane.fontBox:SetPoint("TOPRIGHT", pane.iconBox, "BOTTOMRIGHT", 0, 5)
	pane.fontBox:SetItems(bCastFonts)
	pane.fontText:SetHeight(pane.fontBox:GetHeight())
	pane.fontBox:SetLayer(2)
	
	-- Font Size
	pane.fontSizeText = UI.CreateFrame("Text", unitName .. "FontSizeText", pane.base)
	pane.fontSizeText:SetText("Font size:")
	pane.fontSizeText:SetPoint("TOPLEFT", pane.fontText, "BOTTOMLEFT", 0, 5)	
	pane.fontSizeSlider = UI.CreateFrame("SimpleSlider", unitName .. "FontSizeSlider", pane.base)
	pane.fontSizeSlider:SetPoint("TOPRIGHT", pane.fontBox, "BOTTOMRIGHT", 0, 5)
	pane.fontSizeSlider:SetRange(3, 72)
	pane.fontSizeText:SetHeight(pane.fontSizeSlider:GetHeight())
	pane.fontSizeSlider:SetLayer(1)
	
	-- Font color
	pane.fontColorText = UI.CreateFrame("Text", unitName .. "FontColorLabel", pane.base)
	pane.fontColorText:SetText("Font color:")
	pane.fontColorText:SetPoint("TOPLEFT", pane.fontSizeText, "BOTTOMLEFT", 0, 5)
	pane.fontColorTextRed = UI.CreateFrame("Text", unitName .. "FontColorRedLabel", pane.base)
	pane.fontColorTextRed:SetText("Red")
	pane.fontColorTextRed:SetPoint("TOPLEFT", pane.fontColorText, "BOTTOMLEFT", 0, 5)
	pane.fontColorSliderRed = UI.CreateFrame("SimpleSlider", unitName .. "FontRedSlider", pane.base)
	pane.fontColorSliderRed:SetPoint("TOPRIGHT", pane.fontSizeSlider, "BOTTOMRIGHT", 0, pane.fontColorText:GetHeight() + 10) -- empty space right above this, need to anchor to previous slider above that
	pane.fontColorSliderRed:SetRange(0,255)
	pane.fontColorTextRed:SetHeight(pane.fontColorSliderRed:GetHeight())
	pane.fontColorTextGreen = UI.CreateFrame("Text", unitName .. "FontColorGreenLabel", pane.base)
	pane.fontColorTextGreen:SetText("Green")
	pane.fontColorTextGreen:SetPoint("TOPLEFT", pane.fontColorTextRed, "BOTTOMLEFT", 0, 5)
	pane.fontColorSliderGreen = UI.CreateFrame("SimpleSlider", unitName .. "FontGreenSlider", pane.base)
	pane.fontColorSliderGreen:SetPoint("TOPRIGHT", pane.fontColorSliderRed, "BOTTOMRIGHT", 0, 5)
	pane.fontColorSliderGreen:SetRange(0,255)
	pane.fontColorTextGreen:SetHeight(pane.fontColorSliderGreen:GetHeight())
	pane.fontColorTextBlue = UI.CreateFrame("Text", unitName .. "FontColorBlueLabel", pane.base)
	pane.fontColorTextBlue:SetText("Blue")
	pane.fontColorTextBlue:SetPoint("TOPLEFT", pane.fontColorTextGreen, "BOTTOMLEFT", 0, 5)
	pane.fontColorSliderBlue = UI.CreateFrame("SimpleSlider", unitName .. "FontBlueSlider", pane.base)
	pane.fontColorSliderBlue:SetPoint("TOPRIGHT", pane.fontColorSliderGreen, "BOTTOMRIGHT", 0, 5)
	pane.fontColorSliderBlue:SetRange(0,255)
	pane.fontColorTextBlue:SetHeight(pane.fontColorSliderBlue:GetHeight())
	pane.fontColorSliderRed:SetLayer(1)
	pane.fontColorSliderGreen:SetLayer(1)
	pane.fontColorSliderBlue:SetLayer(1)
	
	-- Ability name length
	pane.lengthText = UI.CreateFrame("Text", unitName .. "LengthText", pane.base)
	pane.lengthText:SetText("Max name length:")
	pane.lengthText:SetPoint("TOPLEFT", pane.fontColorTextBlue, "BOTTOMLEFT", 0, 5)	
	pane.lengthSlider = UI.CreateFrame("SimpleSlider", unitName .. "LengthSlider", pane.base)
	pane.lengthSlider:SetPoint("TOPRIGHT", pane.fontColorSliderBlue, "BOTTOMRIGHT", 0, 5)
	pane.lengthSlider:SetRange(5, 125)
	pane.lengthText:SetHeight(pane.lengthSlider:GetHeight())
	pane.lengthSlider:SetLayer(1)
	
	-- Bar enabler
	pane.enabledText = UI.CreateFrame("Text", unitName .. "EnabledText", pane.base)
	pane.enabledText:SetText("Bar enabled:")
	pane.enabledText:SetPoint("TOPLEFT", pane.lengthText, "BOTTOMLEFT", 0, 5)	
	pane.enabledBox = UI.CreateFrame("RiftCheckbox", unitName .. "EnabledBox", pane.base)
	pane.enabledBox:SetPoint("TOPRIGHT", pane.lengthSlider, "BOTTOMRIGHT", -240, 5)
	pane.enabledBox:SetHeight(pane.enabledText:GetHeight())
	pane.enabledBox:SetLayer(1)
	
	if unitName == "player" then
		-- Pushback
		pane.pushbackText = UI.CreateFrame("Text", unitName .. "PushbackText", pane.base)
		pane.pushbackText:SetText("Show pushback:")
		pane.pushbackText:SetPoint("TOPLEFT", pane.enabledText, "BOTTOMLEFT", 0, 5)	
		pane.pushbackBox = UI.CreateFrame("RiftCheckbox", unitName .. "PushbackBox", pane.base)
		pane.pushbackBox:SetPoint("TOPRIGHT", pane.enabledBox, "BOTTOMRIGHT", 0, 5)
		pane.pushbackBox:SetHeight(pane.pushbackText:GetHeight())
		pane.pushbackBox:SetLayer(1)
		
		-- GCD disply
		pane.gcdText = UI.CreateFrame("Text", unitName .. "gcdText", pane.base)
		pane.gcdText:SetText("Show GCD:")
		pane.gcdText:SetPoint("TOPLEFT", pane.pushbackText, "BOTTOMLEFT", 0, 5)
		pane.gcdBox = UI.CreateFrame("SimpleSelect", unitName .. "gcdBox", pane.base)
		pane.gcdBox:SetBorder(1,1,1,1,1)
		pane.gcdBox:ResizeToDefault()
		pane.gcdBox:SetWidth(pane.colorSliderBlue:GetWidth())
		pane.gcdBox:SetPoint("TOPRIGHT", pane.pushbackBox, "BOTTOMRIGHT", 240, 5)
		pane.gcdBox:SetItems({"above","below","none"})
		pane.gcdText:SetHeight(pane.gcdBox:GetHeight())
		pane.gcdBox:SetLayer(1)
	else
		-- for target, targettarget and focus
		-- Uninterruptible bar color
		pane.intColorText = UI.CreateFrame("Text", unitName .. "IntColorLabel", pane.base)
		pane.intColorText:SetText("Uninterruptible bar color:")
		pane.intColorText:SetPoint("TOPLEFT", pane.enabledText, "BOTTOMLEFT", 0, 5)
		pane.intColorTextRed = UI.CreateFrame("Text", unitName .. "IntColorRedLabel", pane.base)
		pane.intColorTextRed:SetText("Red")
		pane.intColorTextRed:SetPoint("TOPLEFT", pane.intColorText, "BOTTOMLEFT", 0, 5)
		pane.intColorSliderRed = UI.CreateFrame("SimpleSlider", unitName .. "IntRedSlider", pane.base)
		pane.intColorSliderRed:SetPoint("TOPRIGHT", pane.enabledBox, "BOTTOMRIGHT", 240, pane.intColorText:GetHeight() + 10) -- empty space right above this, need to anchor to previous slider above that
		pane.intColorSliderRed:SetRange(0,255)
		pane.intColorTextRed:SetHeight(pane.intColorSliderRed:GetHeight())
		pane.intColorTextGreen = UI.CreateFrame("Text", unitName .. "IntColorGreenLabel", pane.base)
		pane.intColorTextGreen:SetText("Green")
		pane.intColorTextGreen:SetPoint("TOPLEFT", pane.intColorTextRed, "BOTTOMLEFT", 0, 5)
		pane.intColorSliderGreen = UI.CreateFrame("SimpleSlider", unitName .. "IntGreenSlider", pane.base)
		pane.intColorSliderGreen:SetPoint("TOPRIGHT", pane.intColorSliderRed, "BOTTOMRIGHT", 0, 5)
		pane.intColorSliderGreen:SetRange(0,255)
		pane.intColorTextGreen:SetHeight(pane.intColorSliderGreen:GetHeight())
		pane.intColorTextBlue = UI.CreateFrame("Text", unitName .. "IntColorBlueLabel", pane.base)
		pane.intColorTextBlue:SetText("Blue")
		pane.intColorTextBlue:SetPoint("TOPLEFT", pane.intColorTextGreen, "BOTTOMLEFT", 0, 5)
		pane.intColorSliderBlue = UI.CreateFrame("SimpleSlider", unitName .. "IntBlueSlider", pane.base)
		pane.intColorSliderBlue:SetPoint("TOPRIGHT", pane.intColorSliderGreen, "BOTTOMRIGHT", 0, 5)
		pane.intColorSliderBlue:SetRange(0,255)
		pane.intColorTextBlue:SetHeight(pane.intColorSliderBlue:GetHeight())
		pane.intColorSliderRed:SetLayer(1)
		pane.intColorSliderGreen:SetLayer(1)
		pane.intColorSliderBlue:SetLayer(1)
	end
	
	return pane
end

-- loads the settings from saved variables to show 
function bCastOptionsPane:LoadOptions()
	local config = bCast_profiles
	local curProf = bCast_currentProfile

	self.widthSlider:SetPosition(math.floor(config[curProf][self.unit].width + 0.5))
	self.heightSlider:SetPosition(math.floor(config[curProf][self.unit].height + 0.5))
	self.xPosSlider:SetRange(0, math.floor(UIParent:GetWidth() + 0.5))
	self.xPosSlider:SetPosition(math.floor(config[curProf][self.unit].xPos + 0.5))
	self.yPosSlider:SetRange(0, math.floor(UIParent:GetHeight() + 0.5))
	self.yPosSlider:SetPosition(math.floor(config[curProf][self.unit].yPos + 0.5))
	self.colorSliderRed:SetPosition(math.floor(config[curProf][self.unit].colors.bar_r*255 + 0.5))
	self.colorSliderGreen:SetPosition(math.floor(config[curProf][self.unit].colors.bar_g*255 + 0.5))
	self.colorSliderBlue:SetPosition(math.floor(config[curProf][self.unit].colors.bar_b*255 + 0.5))
	self.texBox:SetSelectedItem(config[curProf][self.unit].texture)
	self.iconBox:SetSelectedItem(config[curProf][self.unit].icon)
	self.fontBox:SetSelectedItem(config[curProf][self.unit].font)
	self.fontSizeSlider:SetPosition(config[curProf][self.unit].fontSize)
	self.fontColorSliderRed:SetPosition(math.floor(config[curProf][self.unit].colors.text_r*255 + 0.5))
	self.fontColorSliderGreen:SetPosition(math.floor(config[curProf][self.unit].colors.text_g*255 + 0.5))
	self.fontColorSliderBlue:SetPosition(math.floor(config[curProf][self.unit].colors.text_b*255 + 0.5))
	self.lengthSlider:SetPosition(config[curProf][self.unit].maxLength)
	self.enabledBox:SetChecked(config[curProf][self.unit].enabled)
	if self.unit == "player" then
		self.pushbackBox:SetChecked(config[curProf][self.unit].pushBack)
		self.gcdBox:SetSelectedItem(config[curProf][self.unit].showGCD)
	else
		self.intColorSliderRed:SetPosition(math.floor(config[curProf][self.unit].colors.bar_ir*255 + 0.5))
		self.intColorSliderGreen:SetPosition(math.floor(config[curProf][self.unit].colors.bar_ig*255 + 0.5))
		self.intColorSliderBlue:SetPosition(math.floor(config[curProf][self.unit].colors.bar_ib*255 + 0.5))
	end
end

-- Updates the position sliders after using the "toggle lock" button to move the cast bars
function bCastOptionsPane:LoadPosition()
	local config = bCast_profiles
	local curProf = bCast_currentProfile
	self.xPosSlider:SetRange(0, math.floor(UIParent:GetWidth() + 0.5))
	self.xPosSlider:SetPosition(math.floor(config[curProf][self.unit].xPos + 0.5))
	self.yPosSlider:SetRange(0, math.floor(UIParent:GetHeight() + 0.5))	
	self.yPosSlider:SetPosition(math.floor(config[curProf][self.unit].yPos + 0.5))
end

-- saves the currently selected values to saved variables
function bCastOptionsPane:SaveOptions()
	local config = bCast_profiles
	local curProf = bCast_currentProfile

	config[curProf][self.unit].width = self.widthSlider:GetPosition()
	config[curProf][self.unit].height = self.heightSlider:GetPosition()
	config[curProf][self.unit].xPos = self.xPosSlider:GetPosition()
	config[curProf][self.unit].yPos = self.yPosSlider:GetPosition()
	config[curProf][self.unit].colors.bar_r = self.colorSliderRed:GetPosition()/255
	config[curProf][self.unit].colors.bar_g = self.colorSliderGreen:GetPosition()/255
	config[curProf][self.unit].colors.bar_b = self.colorSliderBlue:GetPosition()/255
	config[curProf][self.unit].texture = self.texBox:GetSelectedItem()
	config[curProf][self.unit].icon = self.iconBox:GetSelectedItem()
	config[curProf][self.unit].font = self.fontBox:GetSelectedItem()
	config[curProf][self.unit].fontSize = self.fontSizeSlider:GetPosition()
	config[curProf][self.unit].colors.text_r = self.fontColorSliderRed:GetPosition()/255
	config[curProf][self.unit].colors.text_g = self.fontColorSliderGreen:GetPosition()/255
	config[curProf][self.unit].colors.text_b = self.fontColorSliderBlue:GetPosition()/255
	config[curProf][self.unit].maxLength = self.lengthSlider:GetPosition()
	config[curProf][self.unit].enabled = self.enabledBox:GetChecked()
	
	if self.unit == "player" then
		config[curProf][self.unit].pushBack = self.pushbackBox:GetChecked()
		config[curProf][self.unit].showGCD = self.gcdBox:GetSelectedItem()
	else
		config[curProf][self.unit].colors.bar_ir = self.intColorSliderRed:GetPosition()/255
		config[curProf][self.unit].colors.bar_ig = self.intColorSliderGreen:GetPosition()/255
		config[curProf][self.unit].colors.bar_ib = self.intColorSliderBlue:GetPosition()/255
	end
end

function bCastOptionsPane:SetVisible(value)
	self.base:SetVisible(value)
end